@@ -167,9 +167,11 @@ struct tree_descender
 	STEME_TYPEDEF_SEQAN_TYPES( Spec )
 	typedef data< Spec >                                                 data_t;                    ///< Data type.
 	typedef model< Spec >                                                model_t;                   ///< Model type.
-	typedef typename model_t::bs_model_t                                 bs_model_t;                ///< Binding site model type.
-	typedef typename model_t::bs_model_t::partial_evaluation_t           bs_partial_evaluation_t;   ///< The type that stores partial evaluations by the binding site model.
-	typedef optional_pair< bs_partial_evaluation_t >                     partial_evaluation_pair;   ///< A pair of optional partial evaluations by the binding site model.
+    typedef typename model_t::bs_model_t                                bs_model_t;                ///< Binding site model type.
+    typedef typename model_t::bg_model_t                                bg_model_t;                ///< Background model type.
+    typedef typename model_t::bs_model_t::partial_evaluation_t          bs_partial_evaluation_t;   ///< The type that stores partial evaluations by the binding site model.
+    typedef typename model_t::bg_model_t::partial_evaluation_t          bg_partial_evaluation_t;   ///< The type that stores partial evaluations by the background model.
+    typedef optional_pair< bs_partial_evaluation_t >                     partial_evaluation_pair;   ///< A pair of optional partial evaluations by the binding site model.
 
 	data_t &               _data;                   ///< The data.
 	model_t &              _model;                  ///< The model of our data.
@@ -203,6 +205,7 @@ struct tree_descender
 		visit_node(
 			top_down_it( _data.index ),
 			partial_evaluation_pair( bs_model_t::empty_partial_evaluation(), bs_model_t::empty_partial_evaluation() ),
+			bg_model_t::empty_partial_evaluation(),
 			0
 		);
 
@@ -221,6 +224,7 @@ struct tree_descender
 	visit_node(
 		top_down_it it,
 		partial_evaluation_pair partial_eval_pair,
+		bg_partial_evaluation_t partial_bg_eval,
 		size_t previous_w
 	) {
 		using namespace seqan;
@@ -235,14 +239,17 @@ struct tree_descender
 			return;
 		}
 
-		const size_t w = std::min( _data.get_rep_length( it ), W );
+		const size_t w = std::min( repLength( it ), W );
 
 		MYRRH_ASSERT( w > previous_w || 0 == previous_w );
 		const size_t num_occurrences = countOccurrences( it );
 		const prefix_t pre = prefix( representative( it ), w );
 
+		// update our partial evaluation of the background model
+		partial_bg_eval = _model.bg.update_partial_eval( pre, partial_bg_eval, previous_w );
+
 		// get a bound on the evaluation under the background model
-		const double bg_log_likelihood_bound = _model.bg.get_min_log_likelihood( it );
+		const double bg_log_likelihood_bound = _model.bg.get_min_log_likelihood( it, partial_bg_eval, w, W );
 
 		// get a bound on log(1-g)
 		const double weight_bound = _model._prior._psp->get_term_bound( it );
@@ -307,7 +314,7 @@ struct tree_descender
 			} else {
 
 				// can be over-ridden in derived class.
-				static_cast< Derived * >( this )->descend( it, partial_eval_pair, w );
+				static_cast< Derived * >( this )->descend( it, partial_eval_pair, partial_bg_eval, w );
 
 			}
 		}
@@ -322,13 +329,14 @@ struct tree_descender
 	descend(
 		top_down_it it,
 		partial_evaluation_pair partial_eval_pair,
+        bg_partial_evaluation_t partial_bg_eval,
 		size_t w
 	) {
 		// continue descending the suffix tree, so go down and across
 		if( goDown( it ) ) {
-			visit_node( it, partial_eval_pair, w );
+			visit_node( it, partial_eval_pair, partial_bg_eval, w );
 			while( goRight( it ) ) {
-				visit_node( it, partial_eval_pair, w );
+				visit_node( it, partial_eval_pair, partial_bg_eval, w );
 			}
 		}
 	}
