@@ -262,7 +262,7 @@ struct start_finder
 				return;
 			}
 
-			const size_t w = std::min( _data.get_rep_length( it ), W );
+			const size_t w = std::min( repLength( it ), W );
 
 			// did we get to the bottom?
 			if( W == w ) {
@@ -421,7 +421,7 @@ struct start_finder
 			return;
 		}
 
-		const size_t w = std::min( _data.get_rep_length( it ), W );
+		const size_t w = std::min( repLength( it ), W );
 
 		// did we get to the bottom?
 		if( W == w ) {
