@@ -20,7 +20,9 @@ def version_string():
 
 
 from ._stempy_build import *
-from ._stempy_build import _dummy_fn, _debug, _using_old_bg_model, _python_debug_build
+from ._stempy_build import _dummy_fn, _debug, _python_debug_build, _has_google_profiler
+if _has_google_profiler:
+    from ._stempy_build import __google_profiler_start, __google_profiler_stop
 from ._meme_like_output import MemeLikeOutput
 from ._minimal_meme_output import MinimalMemeOutput
     
@@ -28,14 +30,15 @@ from cookbook.named_tuple import namedtuple
 from cookbook.timer import SimpleTimer, Timer
 from cookbook.interval import Interval
 from cookbook.script_basics import log_exception, setup_logging
+from cookbook.pylab_utils import pylab_ioff, pylab_context_ioff
 
-import glob, os, math, numpy as N, itertools, time, subprocess, pylab as P, copy
+import glob, os, math, numpy as N, itertools, time, subprocess, pylab as P, copy, warnings
 from optparse import OptionParser, OptionGroup
 from itertools import imap, chain
 from collections import defaultdict
 
 logger = logging.getLogger(__name__)
-
+logging.captureWarnings(True)
 
 
 
@@ -485,7 +488,7 @@ def log_memory_usage():
     logger.info(check_output(['ps', 'v', '-p', str(os.getpid())]))
 
 
-def logo(dist, tag, d, make_png=False, make_eps=True, write_title=True):
+def logo(dist, tag, d, make_png=False, make_eps=True, write_title=True, show_fineprint=True):
     "Generate a logo with the given tag in the given directory."
     import weblogolib as W
     import corebio.seq
@@ -498,7 +501,7 @@ def logo(dist, tag, d, make_png=False, make_eps=True, write_title=True):
         color_scheme=W.colorscheme.nucleotide,
         show_xaxis=False,
         show_yaxis=False,
-        show_fineprint=True,
+        show_fineprint=show_fineprint,
         fineprint='found by STEME %s' % __version__,
     )
     format_ = W.LogoFormat(data, options)
@@ -552,6 +555,17 @@ def occurrences_from_index(index):
     return freqs
 
 
+def log_composition(occs):
+    """Log the composition of the sequences based on the occurrence counts.
+    """
+    logger.info('%.1f%% of nucleotides unknown, rest distributed as:', float(occs[4] / occs.sum() * 100))
+    logger.info('A: %4.1f%%', float(occs[0] / occs[:4].sum() * 100))
+    logger.info('C: %4.1f%%', float(occs[1] / occs[:4].sum() * 100))
+    logger.info('G: %4.1f%%', float(occs[2] / occs[:4].sum() * 100))
+    logger.info('T: %4.1f%%', float(occs[3] / occs[:4].sum() * 100))
+    logger.info('GC content: %.1f%%', float(occs[1:3].sum() / occs[:4].sum() * 100))
+
+
 def print_theta(theta):
     for col in theta:
         print ' '.join('%.6f' % t for t in col)
@@ -573,7 +587,6 @@ def IC(freqs):
 def check_google_profiling(options):
     "Check that it makes sense to use google profiling if it is turned on."
     if _python_debug_build and options.google_profile:
-        import warnings
         warnings.warn('Cannot use google profiler with debug build. Turning profiler off.')
         options.google_profile = False
 
@@ -581,8 +594,9 @@ def check_google_profiling(options):
 def turn_on_google_profiling_if_asked_for(options):
     "Start google profiling if asked to in the options."
     if options.google_profile:
-        logger.info('Starting google profiler.')
-        __google_profiler_start(os.path.join(options.output_dir, 'google.prof'))
+        profile_file = os.path.join(options.output_dir, 'google.prof')
+        logger.info('Starting google profiler. Writing to: %s', profile_file)
+        __google_profiler_start(profile_file)
 
 
 def turn_off_google_profiling_if_asked_for(options):
@@ -702,6 +716,46 @@ def calc_pssm_cdf(scaled_lo, range_, background):
     return pdf.clip(0, 1.);
 
 
+def create_log_odds(background, pssm, roundto=0, log_odds_range=100):
+    """Create log odds matrix for p-values
+    """
+    log_odds = make_log_odds(pssm=pssm, background=background)
+    scaled_log_odds = scale_log_odds(log_odds, range_=log_odds_range)
+    if roundto:
+        background = background.round(roundto) # round background so results same as MEME (and MAST)
+    pssm_cdf = calc_pssm_cdf(scaled_log_odds, range_=log_odds_range, background=background)
+    return scaled_log_odds, pssm_cdf
+
+
+
+InstanceInfo = namedtuple('InstanceInfo', 'instance seqname seq pos wmer score pvalue')
+"An instance with some extra info about the W-mer and its position."
+
+def instance_info(instance, data, ids, W, scaled_log_odds=None, pssm_cdf=None):
+    """Returns a tuple of information about the instance.
+    """
+    seq, pos = data.pos_localise(instance.global_pos)
+    wmer = data.get_W_mer(W, instance.global_pos)
+    if instance.rev_comp:
+        wmer = reverse_complement(wmer)
+    if None != scaled_log_odds and None != pssm_cdf:
+        score = score_pssm_on_seq(scaled_log_odds, imap(index_for_base, wmer))
+        pvalue = pssm_cdf[score]
+    else:
+        score = pvalue = None
+    return InstanceInfo(
+        instance=instance,
+        seqname=ids[seq],
+        seq=seq,
+        pos=pos,
+        wmer=wmer,
+        score=score,
+        pvalue=pvalue,
+    )
+
+
+
+
 def cmp_predictions(pred1, pred2):
     "Compare 2 predictions."
     seq_idx1, _seq_id1, _seq_len1, start1, site1 = pred1
@@ -733,8 +787,15 @@ def _read_fasta(fasta):
         seq_dict[seq] = record
         seq_indices[seq] = idx
         seq_names[idx] = seq
-        num_bases += len(record)
+        seq_length = len(record)
+        num_bases += seq_length
+        if seq_length > index_max_seq_length:
+            warnings.warn('Sequence too long for index type: max=%d' % index_max_seq_length)
     logger.info('Read %d bases from %d sequences in %s', num_bases, len(seq_dict), fasta)
+    if seq_length > index_max_seqs:
+        warnings.warn('Too many sequences for index type: max=%d' % index_max_seqs)
+    if num_bases > index_max_text_length:
+        warnings.warn('Text too long for index type: max=%d' % index_max_text_length)
     return seq_dict, seq_indices, seq_names
 
 
@@ -841,6 +902,7 @@ def log_stats(stats):
 
 
 
+@pylab_ioff
 def test_matplotlib():
     "Test matplotlib save as EPS problem."
     import pylab as P
@@ -859,11 +921,10 @@ def test_matplotlib():
     P.close()
 
 
+@pylab_ioff
 def graph_stats(stats, tag):
-    "Graph the statistics."
-    
+    "Graph the statistics."    
     import pylab as P
-    
     # fraction evaluated plot
     P.figure()
     width=.4
@@ -972,11 +1033,13 @@ def compare_instances(instances1, W1, instances2, W2):
     return overlap, size1, size2
 
 
+
+
 class StartingPointFinder(object):
     "Finds best starting points for EM."
 
 
-    def __init__(self, input_sequences, significance, options, output_dir=None):
+    def __init__(self, input_sequences, bg_manager, significance, options, output_dir=None):
         "Construct."
         
         self.options = options
@@ -991,6 +1054,8 @@ class StartingPointFinder(object):
         self.input_sequences = input_sequences
         "The input sequences."
         
+        self.bg_manager = bg_manager
+        
         self.significance = significance
         "The significance calculator."
         
@@ -1032,12 +1097,12 @@ class StartingPointFinder(object):
         if W not in self.start_counts:
             self.start_counts[W] = create_start_counter(self.input_sequences.data, W)()
         return self.start_counts[W]
-
-
+    
+    
     def _create_start_finder(self, W):
         "@return: start_finder"
         logger.debug('Creating start finder')
-        model = self.input_sequences.create_model(W)
+        model = self.input_sequences.create_model(self.bg_manager.get_bg_model(W), W)
         model.bs.seed_pseudo_counts = self.options.starts_seed_pseudo_counts
         start_finder = FindStarts(
             model.data,
@@ -1204,6 +1269,7 @@ class MotifFinder(object):
     def __init__(
         self, 
         input_sequences,
+        bg_manager,
         significance,
         ids,
         motif_idx,
@@ -1214,6 +1280,8 @@ class MotifFinder(object):
         self.input_sequences = input_sequences
         "The input_sequences we are finding the motif in."
         
+        self.bg_manager = bg_manager
+        
         self.significance = significance
         "Calculates the significance of motifs."
         
@@ -1486,7 +1554,7 @@ class MotifFinder(object):
         return instance_finder.instances
 
     
-    
+    @pylab_ioff
     def _plot_num_sites_graph(self, start_results):
         """
         Plot information about how number of sites has changed for each start
@@ -1500,7 +1568,7 @@ class MotifFinder(object):
             ])
         P.gca().get_xaxis().set_ticklabels(('start', 'EM', 'discretise'))
         P.gca().get_xaxis().set_ticks((0, 1, 2))
-        P.ylabel('# sites')
+        P.ylabel('\# sites')
         num_sites_graph_filename = os.path.join(self._output_dir_for_motif(), 'num-sites')
         logger.info('Saving number of sites graph to %s', num_sites_graph_filename)
         P.savefig('%s.eps' % num_sites_graph_filename)
@@ -1508,6 +1576,7 @@ class MotifFinder(object):
         P.close()
         
         
+    @pylab_ioff
     def _plot_coverage_vs_E_value(self, start_results):
         """
         Plot how coverage of the start instances changes for the final discretised E-value
@@ -1564,7 +1633,7 @@ class MotifFinder(object):
                     
         logger.info('Ran EM on %d starts', len(start_results))
         self._plot_num_sites_graph(start_results)
-        self._plot_coverage_vs_E_value(start_results)
+        # self._plot_coverage_vs_E_value(start_results) # don't plot as sometimes causes exception (need to investigate)
         
         
         #
@@ -1589,12 +1658,11 @@ class MotifFinder(object):
         # 
         # Create log odds matrix for p-values
         # 
-        background = N.array([best_model.bg.freqs.freq(i) for i in xrange(4)])
-        log_odds = make_log_odds(pssm=N.exp(best_model.bs.pssm.log_probs.values()), background=background)
-        log_odds_range = 100
-        scaled_log_odds = scale_log_odds(log_odds, range_=log_odds_range)
-        rounded_back = background.round(3) # round background so results same as MEME (and MAST)
-        pssm_cdf = calc_pssm_cdf(scaled_log_odds, range_=log_odds_range, background=rounded_back)
+        scaled_log_odds, pssm_cdf = create_log_odds(
+            self.bg_freqs, 
+            N.exp(best_model.bs.pssm.log_probs.values()),
+            roundto=3
+        )
 
 
         #
@@ -1625,7 +1693,7 @@ class MotifFinder(object):
         #
         bg_stats = None
         if self.bg_sequences:
-            bg_prediction_model = self.bg_sequences.create_model(best_model.W)
+            bg_prediction_model = self.bg_sequences.create_model(self.bg_manager.get_bg_model(best_model.W), best_model.W)
             bg_prediction_model.bs.pssm.log_probs.values()[:] = best_model.bs.pssm.log_probs.values()
             bg_prediction_model.bs.recalculate()
             bg_prediction_model.lambda_ = best_model.lambda_
@@ -1673,12 +1741,13 @@ class MotifFinder(object):
         #
         # plot log likelihoods
         #
-        P.figure()
-        P.plot(best.em_result.LLs)
-        P.ylabel('log likelihood')
-        P.xlabel('iteration')
-        P.savefig(os.path.join(self._output_dir_for_motif(), 'LLs.png'))
-        P.close()
+        with pylab_context_ioff():
+            P.figure()
+            P.plot(best.em_result.LLs)
+            P.ylabel('log likelihood')
+            P.xlabel('iteration')
+            P.savefig(os.path.join(self._output_dir_for_motif(), 'LLs.png'))
+            P.close()
         
         # Don't retain all the start results unless explicitly asked
         # as they can occupy a lot of memory
@@ -1756,8 +1825,91 @@ class MotifFinder(object):
             
         return avg_distance, predictions
         
+
+
+def _bg_model_creator_fn(options):
+    """Returns a function to make a background Markov model for the order specified in the options.
+    """
+    try:
+        logger.info('Using a Markov model of order %d' % options.bg_model_order)
+        return globals()['create_markov_model_order_from_index_%d' % options.bg_model_order]
+    except AttributeError:
+        raise ValueError('STEME does not support Markov background models of order %d' % options.bg_model_order)
+
+
+
+def _create_bg_model_from_Markov_fn(options):
+    """Returns a function to make a background Markov model for the order specified in the options.
+    """
+    try:
+        logger.info('Creating a background model from a Markov model of order %d' % options.bg_model_order)
+        return globals()['create_bg_model_from_Markov_model_%d' % options.bg_model_order]
+    except AttributeError:
+        raise ValueError('STEME does not support Markov background models of order %d' % options.bg_model_order)
+
+
+
+def build_Markov_model_of(index, options):
+    """Build a Markov model of the sequences.
+    """
+    return _bg_model_creator_fn(options)(index, options.back_dist_prior)
+    
+
         
 
+class BgModelManager(object):
+    """Manages background models.
+    """
+    
+    def __init__(self, sequences, mm, options):
+        self.sequences = sequences
+        self.mm = mm
+        self.options = options
+    
+        
+
+
+
+class LogLikelihoodBgModelManager(BgModelManager):
+    """Creates background models for various widths based on log likelihoods.
+    """
+    
+    def __init__(self, sequences, mm, options):
+        super(LogLikelihoodBgModelManager, self).__init__(sequences, mm, options)
+        logger.info('Calculating likelihoods of %d bases.', self.sequences.data.N)
+        self.lls = self.mm.calculate_likelihoods(self.sequences.data)
+        self.bg_models = {}
+
+
+    def get_bg_model(self, W):
+        """
+        Return a background model of the sequences for the specified width.
+        """
+        if W not in self.bg_models:
+            self.bg_models[W] = create_bg_model_from_base_likelihoods(
+                W, self.sequences.data, 
+                self.lls, 
+                self.sequences.freqs_with_pseudo_counts
+            )
+        return self.bg_models[W]
+
+
+
+
+class MarkovBgModelManager(BgModelManager):
+    """Creates background models using a Markov model.
+    """
+    
+    def __init__(self, sequences, mm, options):
+        super(MarkovBgModelManager, self).__init__(sequences, mm, options)
+        self.bg_model = _create_bg_model_from_Markov_fn(options)(self.mm) # we can use same background model for all widths
+
+    def get_bg_model(self, W):
+        """
+        Return a background model of the sequences for the specified width.
+        """
+        return self.bg_model
+
 
 
 
@@ -1769,46 +1921,13 @@ class SequenceSet(object):
     def __init__(self, fasta_filename, options):
         self.options = options
         self.num_bases, self.seqs, self.ids, self.index = read_sequences(fasta_filename, self.options)
-        # self.index = build_index(self.seqs) # already done in read_sequences()
-        self.occs = occurrences_from_index(self.index)
+        self.data = Data(self.index, max_W = self.options.max_w)
+        self.occs = occurrences_from_index(self.data.index)
         self.freqs = ZeroOrderFrequencies(list(self.occs[:4]))
         self.freqs_with_pseudo_counts = self.freqs.add_pseudo_counts(self.options.back_dist_prior)
-        self.data = Data(self.index, max_W = self.options.max_w)
-        self.bg_models = {}
+        self.mm, _ = build_Markov_model_of(self.data.index, self.options)
     
     
-    def _bg_model_creator_fn(self):
-        """
-        @return: The function to make a background model for the given order.
-        """
-        try:
-            logger.info('Using a Markov model of order %d' % self.options.bg_model_order)
-            return globals()['create_markov_model_order_from_index_%d' % self.options.bg_model_order]
-        except AttributeError:
-            raise ValueError('STEME does not support Markov background models of order %d' % self.options.bg_model_order)
-    
-
-    def build_model_of(self):
-        """
-        Build a Markov model of the sequences.
-        """
-        self.mm, _ = self._bg_model_creator_fn()(self.data.index, self.options.back_dist_prior)
-        return self.mm, self.freqs, self.freqs_with_pseudo_counts
-        
-    
-    def calculate_likelihoods(self, mm):
-        self.lls = mm.calculate_likelihoods(self.data)
-
-
-    def get_bg_model(self, W):
-        """
-        Return a background model of the sequences for the specified width.
-        """
-        if W not in self.bg_models:
-            self.bg_models[W] = create_bg_model_from_base_likelihoods(W, self.data, self.lls, self.freqs_with_pseudo_counts)
-        return self.bg_models[W]
-
-
     def composition(self):
         """
         The composition of the sequences.
@@ -1819,9 +1938,8 @@ class SequenceSet(object):
         return self.num_bases, len(self.seqs), freqs, unknown
     
     
-    def create_model(self, W):
+    def create_model(self, bg, W):
         "@return: A model."
-        bg = self.get_bg_model(W)
         bs = PssmBindingSiteModel(initialise_uniform_pssm(W, self.options.alphabet_size))
         model = Model(self.data, bs, bg, _lambda=self.options.lambda_)
         return model
@@ -1886,7 +2004,11 @@ class Algorithm(object):
         self.bg_models = dict()
         self.outputs = []
         
-        
+
+    def create_model_of_input(self, W):
+        return self.input_sequences.create_model(self.bg_manager.get_bg_model(W), W)
+    
+            
     def _setup_logging(self, level=logging.INFO):
         formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
         file_handler = logging.FileHandler(os.path.join(self.options.output_dir, 'STEME.log'), mode='w')
@@ -1906,20 +2028,22 @@ class Algorithm(object):
         
     def _initialise_bg(self):
         "Initialise a Markov model for the background."
+
         self.bg_sequences = None
         # Was a background FASTA file specified?
         if self.options.bg_fasta_file:
             # yes
             logger.info('Building background Markov model from sequences in %s', self.options.bg_fasta_file)
             self.bg_sequences = SequenceSet(self.options.bg_fasta_file, self.options)
-            self.mm, self.freqs, self.freqs_with_pseudo_counts = self.bg_sequences.build_model_of()
-            self.bg_sequences.calculate_likelihoods(self.mm)
+            self.bg_manager = get_background_manager(self.input_sequences, self.bg_sequences.mm, self.options)
+            self.freqs = self.bg_sequences.freqs
+            self.freqs_with_pseudo_counts = self.bg_sequences.freqs_with_pseudo_counts
         else:
             # no, build background model from sequences we are searching for motifs in
             logger.info('Building background Markov model from input sequences')
-            self.mm, self.freqs, self.freqs_with_pseudo_counts = self.input_sequences.build_model_of()
-        # calculate likelihood of input sequences using background model.
-        self.input_sequences.calculate_likelihoods(self.mm)
+            self.bg_manager = get_background_manager(self.input_sequences, self.input_sequences.mm, self.options)
+            self.freqs = self.input_sequences.freqs
+            self.freqs_with_pseudo_counts = self.input_sequences.freqs_with_pseudo_counts
         
         
     def background_composition(self):
@@ -1949,6 +2073,7 @@ class Algorithm(object):
         "@return: A MotifFinder object"
         return MotifFinder(
             self.input_sequences,
+            self.bg_manager,
             self.significance,
             self.input_sequences.ids, 
             motif_idx,
@@ -2064,13 +2189,13 @@ class Algorithm(object):
         # Report how long various parts took
         #
         for msg, duration in self.timings:
-            logging.info('%20s took %9.1f seconds (%2.0f%% of total) to complete', msg, duration, 100*duration/self.total_duration)
+            logging.info('%20s took %9.1f seconds (%2.0f%% of total)', msg, duration, 100*duration/self.total_duration)
 
 
     def _find_starts(self):
         "Find the starts we will use."
         with Timer(msg='find starts') as timer:
-            self.start_finder = StartingPointFinder(self.input_sequences, self.significance, self.options)
+            self.start_finder = StartingPointFinder(self.input_sequences, self.bg_manager, self.significance, self.options)
             self.start_finder.find_starts()
             self.timings.append((timer._msg, timer.timer.duration()))
 
