@@ -18,12 +18,17 @@ def append_to_path(d):
     logging.info('Appending to sys.path: %s', d)
     sys.path.append(d) # stempy
     
+def prepend_to_path(d):
+    logging.info('Prepending to sys.path: %s', d)
+    sys.path.insert(0, d) # stempy
+    
 def update_path_for_stempy():
     d = os.path.dirname(__file__)
-    append_to_path(os.path.normpath(os.path.join(d, '..', '..'))) # stempy
-    append_to_path(os.path.normpath(os.path.join(d, '..', '..', '..', '..', 'Python', 'Cookbook'))) # cookbook
-    append_to_path(os.path.normpath(os.path.join(d, '..', '..', '..', '..', 'Infpy', 'python'))) # Infpy
-    append_to_path(os.path.normpath(os.path.join(d, '..', '..', '..', '..', 'PyICL', 'Python'))) # PyICL
+    prepend_to_path(os.path.normpath(os.path.join(d, '..', '..', '..', '..', 'Infpy', 'python'))) # Infpy
+    prepend_to_path(os.path.normpath(os.path.join(d, '..', '..', '..', '..', 'PyICL', 'Python'))) # PyICL
+    prepend_to_path(os.path.normpath(os.path.join(d, '..', '..', '..', '..', 'Python', 'Cookbook', 'python'))) # cookbook
+    prepend_to_path(os.path.normpath(os.path.join(d, '..', '..'))) # stempy
+    print sys.path
 
 def fasta_dir():
     return os.path.normpath(os.path.join(os.path.dirname(__file__), 'fasta'))
